/* ***********************************************************************
**
**  Copyright (C) 2005  Christian Kranz
**
**  Siemens S65 Display Control
**
** rewritten in C by Juergen Eckert (2007)
** V0.1
*********************************************************************** */


#include <avr/io.h>
#include <util/delay.h>
#include <avr/pgmspace.h>

#include "lcd.h"
#include "flexport.h"

prog_uint8_t INIT1[24] =
{
0xFD, 0xFD, 0xFD, 0xFD,
0xEF, 0x00, 0xEE, 0x04, 0x1B, 0x04, 0xFE, 0xFE, 0xFE, 0xFE, 0xEF, 0x90, 0x4A, 0x04,
0x7F, 0x3F, 0xEE, 0x04, 0x43, 0x06,
};

prog_uint8_t INIT2[40] = 
{
0xEF, 0x90,  0x09, 0x83,  0x08, 0x00,  0x0B, 0xAF,  0x0A, 0x00,  0x05, 0x00,  0x06, 0x00,
0x07, 0x00,  0xEF, 0x00,  0xEE, 0x0C,  0xEF, 0x90,  0x00, 0x80,  0xEF, 0xB0,  0x49, 0x02,
0xEF, 0x00,  0x7F, 0x01,  0xE1, 0x81,  0xE2, 0x02,  0xE2, 0x76,  0xE1, 0x83,
};

prog_uint8_t INIT3[2] = 
{
0x80, 0x01
};

void mswait(unsigned int ms)
{
	unsigned int i;
	for(i=0; i<ms; i++)
		_delay_ms(1);
}

void wrtab(unsigned char* data, unsigned char size)
{
	unsigned char i;
	for(i=0; i<size; i++)
	{
		LCD_WrCmd(pgm_read_word_near(&data[i]));
		//LCD_WrCmd(pgm_read_word_far(&data[i]));
	}
}

void LCD_Init(void)
{

	/* Init IO */
	OUTPUT(LCD_RESET);				//set reset as output
	CLR(LCD_RESET);					//Display reset

	INPUT(LCD_MISO);				//MISO is always input in SPI
	SET(LCD_MISO);					//enable pull up of MISO to avoid floating input
	OUTPUT(LCD_MOSI);				//MOSI must be set as output for SPI

	OUTPUT(LCD_CS);					//SS must be output for Master mode to work
	SET(LCD_CS);					//deselect display 

	OUTPUT(LCD_SCK);				//set SCK as output
	CLR(LCD_SCK);					//set SCK lo

	OUTPUT(LCD_RS);					//RS signal is output
	SET(LCD_RS);					//start with RS=high

	/* Init SPI */
	//setup serial data interface 
	//select clock phase positive going in middle of data
	//master mode
	//enable SPI
	//speed is CPUclock/2
	SPCR = (1<<MSTR) | (1<<SPE);			//Master Mode,  Enable SPI, fCPU/4 clock
	SPSR |= (1<<SPI2X);				//double speed bit

	mswait(1);
	SET(LCD_RESET);					//clear status
	wrtab(INIT1, 24);
	mswait(7);
	wrtab(INIT2, 40);
	mswait(50);
	wrtab(INIT3, 2);

}

void LCD_ClrScr(void)
{
	/* send window write start sequence */
	LCD_WrCmd(0xEF);
	LCD_WrCmd(0x90);
	LCD_WrCmd(0x05);
	LCD_WrCmd(0x00);
	LCD_WrCmd(0x06);
	LCD_WrCmd(0x00);
	LCD_WrCmd(0x07);
	LCD_WrCmd(0x00);

	/* now 132*176*2 bytes are to be transmitted */
	unsigned int i;
	for(i=0; i<46464; i++)
		LCD_WrData(0xFF);
}

void LCD_WrCmd(unsigned char data)
{
	CLR(LCD_CS);					//select Display
	SET(LCD_RS);					//set RS line
	SPDR = data;
	while(!(SPSR & (1<<SPIF)));
	SET(LCD_CS);
}

void LCD_WrData(unsigned char data)
{
	CLR(LCD_CS);					//select Display
	CLR(LCD_RS);					//clear RS line
	SPDR = data;
	while(!(SPSR & (1<<SPIF)));
	SET(LCD_CS);
}
